# -*- coding: utf-8 -*-

# Copyright (c) Pilot Systems, 2013

# This file is part of Faceted SeSQL.

# SeSQL is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# SeSQL is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Faceted SeSQL.  If not, see <http://www.gnu.org/licenses/>.

"""
This is Faceted SeSQL simple benchmarking tool.
"""
import sys
import time
import random
from optparse import make_option

from django.core.management.base import BaseCommand
from django.core import serializers

from faceted_sesql.facets import *
from pizza.models import *
from pizza.facets import pizza
from faceted_sesql.search import FacetedSearch

class Command(BaseCommand):
    help = "Perform benchmarks on Faceted SeSQL (requires existing db)"

    option_list = BaseCommand.option_list + (
        make_option('-s', '--seed',
                    dest='seed',
                    type='int',
                    default=42,
                    help='Random seed (default 42)'),

        make_option('-c', '--criteria',
                    dest='criteria',
                    type='int',
                    default=3,
                    help='Maximal number of criteria (default 3)'),

        make_option('-n', '--nb',
                    dest='nb',
                    type='int',
                    default=1000,
                    help='Number of queries per round (default 1000)'),
        )

    def handle(self, *apps, **options):
        """
        Handle the command
        """
        self.options = options
        random.seed(options['seed'])

        for n in range(self.options['criteria']):
            self.bench_for(n + 1)

    def chose_facet(self, search, done):
        """
        Chose on which facet to filter
        """
        facets = search.data.facets.keys()
        while True:
            facet = random.choice(facets)
            if facet in done:
                if random.randint(1,3) != 1:
                    # Only one chance in 3 to reuse existing facet
                    continue
            break
        done.add(facet)
        return facet

    def chose_value(self, search, facet):
        """
        Chose a value from a facet
        """
        facet = search.data.facets[facet]
        # Check if it's a min/max or a select facet
        if isinstance(facet.facet, RangeFacet):
            def randval():
                return random.randint(facet.min_value, facet.max_value)
            start = min(randval(), randval())
            end = max(randval(), randval())
            return (start, end)
        else:
            nbvals = len(facet.values)
            if not nbvals:
                return None
            nb = min(random.randint(1, nbvals), random.randint(1, nbvals))
            return random.sample(facet.values.keys(), nb)

    def bench_for(self, nbcrit):
        """
        Bench for this number of criteria
        """
        print "Performing %d queries with %d criteria..." % (self.options['nb'], 
                                                             nbcrit)
        start = time.time()
        nb = self.options['nb']
        for i in range(nb):
            search = FacetedSearch(pizza)
            done = set()
            for j in range(nbcrit):
                facet = self.chose_facet(search, done)
                value = self.chose_value(search, facet)
                search.set_criteria(facet, value)
                if len(search) == 0:
                    # Invalid search ? well, stop here
                    break
        end = time.time()
        delta = end - start
        print "Done with %d criteria in %.2f seconds, rate is %.2f" % (nbcrit, delta, nb/delta)
